#/bin/sh

# 5'-3' exonucleases: eubacterial and eubacteriophage 5'-3' exonuclease
#
# A number of proteins share a basic feature: the presence of a N-terminal
# (N) and Internal (I) 5'-3' exonuclease domain (in that order). They
# all exhibit 5'-3' exonuclease activity. Besides this, there are
# several subgroups in this set, and each of them can be characterized
# by a metamotif. These are:
#
# - eubacterial DNA polymerase 1
# - eubacterial and eubacteriophage 5'-3' exonuclease
# - eucaryotic structure-dependent endo- and exonucleases
# - eucaryotic virion host shutoff proteins
# - eucaryotic DNA repair proteins
#
# See http://hits.isb-sib.ch/doc/classification.shtml for more
# information.
#
# What separates the DNA polymerases from other 5'-3' exonucleases is:
# 1) no more than 40 residues between the N and I domains.
# 2) no more than 250 residues between the I domain and the C terminus
# 3) absence of XPG motifs in the N domains (by contrast to the
# structure-dependent nucleases).

cat c_termini.pff domains.pff | mmsearch  \
'^<prf:53EXO_N_DOMAIN = (pat:XPG_1){,0} = prf:53EXO_N_DOMAIN> = ,40 = prf:53EXO_I_DOMAIN = ,250 = C_TERM'

# Expression '(pat:XPG_1){,0}' means that pat:XPG_1 must _not_ occur
# here (literally, that the maximal number of occurrences is zero),
# and it is between the start (<prf:53EXO_N_DOMAIN) and the end
# (prf:53EXO_N_DOMAIN>) of an N domain. In other words: no XPG_1
# inside the N domain.
#
# We use the start-anchor (^) so speed things up, as this motif does
# not occur after anything else in the protein (the N-terminal domain
# would preclude it) Still, the (...){...} counting struture entails a
# slowdown.

