#/bin/sh

# 5'-3' exonucleases: eukaryotic virion host dhutoff proteins
#
# A number of proteins share a basic feature: the presence of a N-terminal
# (N) and Internal (I) 5'-3' exonuclease domain (in that order). They
# all exhibit 5'-3' exonuclease activity. Besides this, there are
# several subgroups in this set, and each of them can be characterized
# by a metamotif. These are:
#
# - eubacterial DNA polymerase 1
# - eubacterial and eubacteriophage 5'-3' exonuclease
# - eucaryotic structure-dependent endo- and exonucleases
# - eucaryotic virion host shutoff proteins
# - eucaryotic DNA repair proteins
#
# See http://hits.isb-sib.ch/doc/classification.shtml for more
# information.
#
# What separates the DNA polymerases from other 5'-3' exonucleases is:
# 1) from 50 to 120 residues between the N and I domains.

mmsearch < domains.pff '^prf:53EXO_N_DOMAIN = 50,120 = prf:53EXO_I_DOMAIN'

# We use the start-anchor (^) so speed things up, as this motif does
# not occur after anything else in the protein (the N-terminal domain
# would preclude it).
#
# In this case the metamotif is not very useful, because Pfam has a
# VIRUS_HS hmm that targets the same sequences. The only advantage is
# if some part outside the metamotif is unrecognizable by the VIRUS_HS
# HMM - say because of a frameshift. In trGen and trEST (see reference
# in README), this is far from uncommon.
