################################################################
#            The mmsearch Metamotif Search Engine
#
# Thomas Junier, April 2001
#
# Please see file COPYRIGHT.
#
# The Automaton class and its subclasses: building blocks of an
# automaton. The complete automaton is built by linking sub-automata
# together (see class Transition). 

import string, re
from mmState import *


class Automaton:
	"""An abstract automaton."""
	def get_start_state(self): return self.start_state
	def get_link_state(self): return self.link_state
	def reset(self):
		for state in state_list:
			state.reset()
	def matches(self, master_input_string, caret='no', dollar='no',
		overlaps='no', quick='no'):
		"""Get all the matches of the FSA in `input_string'."""
		debug = 0
		results = []	
		while master_input_string:
			# input string gets consumed by the automaton, so use a copy:
			input_string = master_input_string[:] 
			# now get the next match:
			if debug: print "\nTry to match..."
			if debug: print "IS (before)", string.join(input_string, '')
			match = self.next_match(input_string)
			if debug: print "IS (after) ", string.join(input_string, '')
			# append to results unless anchored at end ('$')
			# and input isn't empty
			if match != None:
				if debug: print "Match!"
				if dollar == 'yes' and self.more_features(input_string):
					pass 
				else:
					match_string = self.extract_match(input_string,
						master_input_string)
					results.append(match_string)
					# if overlaps aren't allowed, set master to
					# remainder of input -- unless anchored at start
					if overlaps == 'no' and caret == 'no':
						master_input_string = input_string[:]
						continue
			# if anchored at start of string ('^'), don't iterate further
			if caret == 'yes':
				break
			if quick == 'yes':
				# skip to next _unread_ position (i.e., don't try inside
				# a region that was already rejected).
				master_input_string = input_string[:]
			else:
				# skip to the nearest feature
				master_input_string = \
					self.jump_to_next_feature(master_input_string)
			if debug: print "IS (jump)  ", string.join(master_input_string, '')
		return results

	def next_match(self, input_string):
		"""Find the next match the FSA (i.e., self) in
		`input_string', and return it."""
		debug = 0
		# empty backtrack stack
		while len(self.state_stack) > 0: self.state_stack.pop(0) 
		current_state = self.start_state	# reset start state
		while 1:
			if debug: print "\nState: %s" % current_state.label
			if debug: print "Input: '%s'" % string.join(input_string, '')
			if debug: print "Stack: " + `self.state_stack`
			if debug and input_string != []:
				if debug: print '%s: %s ->' % \
					(input_string[0], current_state.label),
			# get the next state, given `input_string'
			next_state = current_state.get_next_state(input_string)
			if next_state != None:	
				# ok, we can proceed
				if debug: print next_state.label
				current_state = next_state
			elif current_state.get_status() == 'accept': 
				# blocked, but in an acceptor: return a match.
				if debug: print "State status: %s" % \
					 current_state.get_status()
				return('match')
			elif len(self.state_stack) > 0:
				# blocked, but we can jump back to other
				# branches we didn't try yet
				current_state = self.state_stack[-1]
				if debug: print "Jumping to top of stack (%s)." % \
					current_state.label
			elif current_state.get_status() == 'reject':
				# blocked, no match was found
				if debug: print 'Nowhere to go.'
				return None
			else:
				# should never get here
				raise RuntimeError, 'Automaton is in an unexpected state.'

	def more_features(self, s):
		"""Returns true iff there are one or more features left in `s' - that
		is, it ignores digits, spaces and '-'s. Used when the meta-motif is
		end-anchored ($)."""
		for c in s:
			if not (c in string.digits or c in (' ', '-')):
				return 1
		return 0

	def jump_to_next_feature(self, s):
		"""Finds the index of the next feature end ('<a', 'c>,
		etc.) in `s', and returns a substring of `s' from that
		position to its end. Used to advance to next possible
		start of match, when looking for multiple matches."""
		# See the format of the string representation of
		# sequences (in the manual) for more details.
		try:
			while s[0] != '-': s.pop(0)	# this feature end's second '-'
			s.pop(0)	# skip it
			while s[0] != '-': s.pop(0)	# next feature end's first '-'
			s.pop(0)	# skip it
			return s
		except IndexError, e:
			# string has no more features
			return ''

	def extract_match(self, i_s, m_i_s):
		"""Returns the part of the input string that matches the
		metamotif."""
		# TODO: This method is too long and inefficient (mainly due to
		# debugging code, though).
		#
		# This function is crucial for the correct representation of
		# the output, so I'll leave a `debug' variable to facilitate
		# is debugging - it is quite tricky. The reason is that the
		# stop and start state of the automaton do not always coincide
		# with feature names (letters, in this case): the machine may,
		# for example, stop on a number. The match string has to be
		# reformatted into a <num>-<end>-<num>[...] format, which is
		# expected by the output functions. Set debug to 1 to see the
		# effects.
		debug = 0 #  don't hesitate to set this to 1 if you have problems...
		if debug: print "\n***** Returning Matches *****"
		if debug: print "*** raw data: ***"
		if debug: print "Input, from start state:", string.join(m_i_s, '')
		if debug: print "Input, from end state:  ", string.join(i_s, '')
		remaining_len = len(string.join(i_s, ''))
		if remaining_len == 0:
			matching_string = string.join(m_i_s, '')
			c_term = re.compile(r' \d+$')
			matching_string = c_term.sub('', matching_string)
		else:
			matching_string = string.join(m_i_s, '')[:-remaining_len]
		match_len = len(string.join(m_i_s, '')) - remaining_len
		if debug: print "*** extracting match string: ***"
		if debug: print "Remaining len:", remaining_len
		if debug: print "Match len:", match_len
		if debug: print "Input, from start state:", string.join(m_i_s, '')
		if debug: print "Input, from end state:  ", match_len * '.' + string.join(i_s, '')
		if debug: print "Match string:           ", matching_string
		if debug: print "*** reformatting match string ***"
		# remove the '#\d+' so as to retain the situation before ft IDs.
		if debug: print "** removing '#\d+' **"
		matching_string = re.compile(r'#\d+').sub('', matching_string)
		if debug: print "Match string:", matching_string
		# if match consists of a sole feature end (i.e., without
		# position), append begin of remaining string (up to first ' ').
		if re.compile(r'\d').search(matching_string) == None:
			if debug: print "** adding position info **"
			pos = re.compile(r'-\d+').search(string.join(i_s, ''))
			matching_string += pos.group(0)
			if debug: print "Match string:", matching_string
		# reform the last feature end: repeat position
		if debug: print "** repeating last position (if needed) **"
		pos_last = re.compile(r'((\d+)-[\w><]+-?)$')
		matching_string = pos_last.sub(r'\1-\2', matching_string)
		if debug: print "Match string:", matching_string
		# when there is a trailing IDchecker, we can have a '--'
		# instead of '-'. Remove it.
		if debug: print "** removing '--' in case of trailing IDchecker **"
		matching_string = re.compile(r'--').sub('-', matching_string)
		if debug: print "Match string:", matching_string
		# remove first number of `matching string', which isn't part
		# of the match (it's the position of the last non-matching
		# feature).
		if debug: print ("** removing position of last non-matching " 
			"feature (if needed) **")
		pos_first = re.compile(r'^-?\d+ ')
		matching_string = pos_first.sub('', matching_string)
		if debug: print "Match string:", matching_string
		# depending on whether overlaps are allowed (-O switch),
		# the matching string now starts either with the first
		# feature end or with its (first) position, i.e.	12-<a-12
		# -O active (ok) <a-12   -O inactive (needs reformatting)
		# (The first match always behave as if -O is active). When
		# -O is inactive, we must then reconstruct the missing
		# number.
		if debug: print "** repeating first position (if needed) **"
		matching_string = re.compile(r'^([\w<>]+-(\d+))').sub(r'\2-\1',
				matching_string)
		if debug: print "Match string:", matching_string
		return matching_string

################################################################
# Subclasses

class Feature_transition(Automaton):
	"""Embodies a feature match transition, e.g. S(a)->E"""
	def __init__(self, state_stack, symbol):
		# the smallest possible fragment
		f = State('end', status='accept')	# new final state
		i = Feature(symbol,symbol=symbol)	# new initial state
		i.set_next_state(f)
		self.start_state = i
		self.link_state = i	# NOT f
		self.state_stack = state_stack


class Any_transition(Automaton):
	"""Embodies an unspecified transition ('x')."""
	def __init__(self, state_stack):
		f = State('end', status='accept')	# new final state
		i = Any(symbol,symbol=symbol)	# new initial state
		i.set_next_state(f)
		self.start_state = i
		self.link_state = i	# NOT f
		self.state_stack = state_stack


class IDsetter_transition(Automaton):
	"""Embodies an ID setter transition."""
	def __init__(self, state_stack, name, symtable):
		f = State('end', status='accept')	# new final state
		i = IDsetter(name + '_setter', name, symtable)
		i.set_next_state(f)
		self.start_state = i
		self.link_state = i	# NOT f
		self.state_stack = state_stack


class IDchecker_transition(Automaton):
	"""Embodies an ID checker transition."""
	def __init__(self, state_stack, name, symtable):
		f = State('end', status='accept')	# new final state
		i = IDchecker(name + '_checker', name, symtable)
		i.set_next_state(f)
		self.start_state = i
		self.link_state = i	# NOT f
		self.state_stack = state_stack


class Spacer_transition(Automaton):
	"""Embodies a spacer transition."""
	def __init__(self, state_stack, label, min, max):
		f = State('epsilon-end(%s)' % label, status='accept')
		i = Spacer(label, min=min, max=max)
		i.set_next_state(f)
		self.start_state = i
		self.link_state = i	# NOT f
		self.state_stack = state_stack


class Transition(Automaton):
	"""Construct an Automaton out of a list of sub-automata, linking
	them in transition. There is an accepting, no-transition state
	at the end."""
	def __init__(self, state_stack, automaton_list):
		self.state_stack = state_stack
		head = automaton_list.pop(0)
		self.start_state = head.get_start_state()
		link = head.get_link_state()
		while automaton_list != []:
			tail = automaton_list.pop(0)
			start = tail.get_start_state()
			link.set_next_state(start)
			link = tail.get_link_state()
		self.link_state = link


class Alternation(Automaton):
	"""Takes n sub-automata in a list (`branches') and links them
	into an alternation structure."""
	def __init__(self, state_stack, label, branches):
		self.state_stack = state_stack
		# build Head and Tail of the alternator
		i = Alternator_Head(state_stack, 'start(%s)' % label)
		f = Alternator_Tail(state_stack, 'end(%s)' % label)
		f.set_head(i)
		# now link all branch starts to `i', and all branch link
		# states to `f'
		for branch in branches:
			branch_start = branch.get_start_state()
			i.set_next_state(branch_start)
			branch_link = branch.get_link_state()
			branch_link.set_next_state(f)
		self.start_state = i
		self.link_state = f


class Counter(Automaton):
	"""Implements a counter around a sub-automaton."""
	def __init__(self, state_stack, label, loop, min, max):
		self.state_stack = state_stack
		ch = Counter_Head(state_stack, 'head(%s)' % label)	
		ct = Counter_Tail(state_stack, 'tail(%s)' % label, min, max)
		ch.set_tail(ct)
		ct.set_head(ch)
		# loop around the `loop' sub-automaton
		ch.set_next_state(loop.get_start_state())
		link = loop.get_link_state()
		link.set_next_state(ch)
		self.start_state = ch
		self.link_state = ct


class Fork(Automaton):
	"""Implements a fork (parallel alternation)."""
	def __init__(self, state_stack, label, branches, type):
		self.state_stack = state_stack
		start_states = []
		for branch in branches:
			start_states.append(branch.get_start_state())
		f = State('end', status='accept')	# new final state
		i = Fork_head(state_stack, 'head(%s)' % label, type)
		i.set_branches(start_states)
		i.set_next_state(f)
		self.start_state = i
		self.link_state = i
