#/bin/sh

# 5'-3' exonucleases: eubacterial DNA polymerases
#
# A number of proteins share a basic feature: the presence of a N-terminal
# (N) and Internal (I) 5'-3' exonuclease domain (in that order). They
# all exhibit 5'-3' exonuclease activity. Besides this, there are
# several subgroups in this set, and each of them can be characterized
# by a metamotif. These are:
#
# - eubacterial DNA polymerase 1
# - eubacterial and eubacteriophage 5'-3' exonuclease
# - eucaryotic structure-dependent endo- and exonucleases
# - eucaryotic virion host shutoff proteins
# - eucaryotic DNA repair proteins
#
# See http://hits.isb-sib.ch/doc/classification.shtml for more
# information.
#
# What separates the eubacterial DNA polymerases from other 5'-3'
# exonucleases is: 1) from 4 to 10 residues (usually 4) between the N and
# I domains 2) at least 600 residues after the end of the I domain. This
# translates as:

cat c_termini.pff domains.pff | mmsearch  \
'prf:53EXO_N_DOMAIN = 4,10 = prf:53EXO_I_DOMAIN = 600, = C_TERM'

# Of course, there also is a DNA polymerase domain near the C
# terminus, but this is also found in eucaryotes, so by itself does
# not help in finding eubacterial sequences. 
